from contextlib import suppress
from datetime import timedelta

from dateutil.parser import parse

from pretalx.person.models import SpeakerProfile, User
from pretalx.schedule.models import Room, TalkSlot
from pretalx.submission.models import (
    Submission, SubmissionStates, SubmissionType,
)


def process_frab(root, event):
    """
    Takes an xml document root and an event, and releases a schedule
    with the data from the xml document.
    """
    for day in root.findall('day'):
        for rm in day.findall('room'):
            room, _ = Room.objects.get_or_create(event=event, name=rm.attrib['name'])
            for talk in rm.findall('event'):
                _create_talk(talk=talk, room=room, event=event)

    schedule_version = root.find('version').text
    event.wip_schedule.freeze(schedule_version, notify_speakers=False)
    schedule = event.schedules.filter(version=schedule_version).first()
    if not schedule:
        raise Exception(f'Could not import "{event.name}" schedule version "{schedule_version}": failed creating schedule release.')

    schedule.talks.update(is_visible=True)
    return f'Successfully imported "{event.name}" schedule version "{schedule_version}".'


def _create_talk(*, talk, room, event):
    date = talk.find('date').text
    start = parse(date + ' ' + talk.find('start').text)
    hours, minutes = talk.find('duration').text.split(':')
    duration = timedelta(hours=int(hours), minutes=int(minutes))
    duration_in_minutes = duration.total_seconds() / 60
    try:
        end = parse(date + ' ' + talk.find('end').text)
    except AttributeError:
        end = start + duration
    sub_type = SubmissionType.objects.filter(
        event=event, name=talk.find('type').text, default_duration=duration_in_minutes
    ).first()

    if not sub_type:
        sub_type = SubmissionType.objects.create(
            name=talk.find('type').text or 'default', event=event, default_duration=duration_in_minutes
        )

    optout = False
    with suppress(AttributeError):
        optout = talk.find('recording').find('optout').text == 'true'

    code = None
    if Submission.objects.filter(code__iexact=talk.attrib['id'], event=event).exists() or not Submission.objects.filter(code__iexact=talk.attrib['id']).exists():
        code = talk.attrib['id']
    elif Submission.objects.filter(code__iexact=talk.attrib['guid'][:16], event=event).exists() or not Submission.objects.filter(code__iexact=talk.attrib['guid'][:16]).exists():
        code = talk.attrib['guid'][:16]
    else:
        code = None

    sub, _ = Submission.objects.get_or_create(
        event=event,
        code=code,
        defaults={'submission_type': sub_type}
    )
    sub.submission_type = sub_type
    sub.title = talk.find('title').text
    sub.description = talk.find('description').text
    if talk.find('subtitle').text:
        sub.description = talk.find('subtitle').text + '\n' + (sub.description or '')
    sub.abstract = talk.find('abstract').text
    sub.content_locale = talk.find('language').text or 'en'
    sub.do_not_record = optout
    sub.state = SubmissionStates.CONFIRMED
    sub.save()

    for person in talk.find('persons').findall('person'):
        user = User.objects.filter(nick=person.text[:60]).first()
        if not user:
            user = User(nick=person.text[:60], name=person.text, email=f'{person.text}@localhost')
            user.save()
            SpeakerProfile.objects.create(user=user, event=event)
        sub.speakers.add(user)

    slot, _ = TalkSlot.objects.get_or_create(
        submission=sub,
        schedule=event.wip_schedule,
        is_visible=True,
    )
    slot.room = room
    slot.is_visible = True
    slot.start = start
    slot.end = end
    slot.save()



from datetime import timedelta
from urllib.parse import unquote

import pytz
from csp.decorators import csp_update
from django.http import Http404, HttpResponse, HttpResponsePermanentRedirect
from django.urls import resolve, reverse
from django.utils.decorators import method_decorator
from django.utils.functional import cached_property
from django.utils.timezone import now
from django.views.generic import TemplateView

from pretalx.common.mixins.views import PermissionRequired
from pretalx.common.signals import register_data_exporters


class ScheduleDataView(PermissionRequired, TemplateView):
    template_name = 'agenda/schedule.html'
    permission_required = 'agenda.view_schedule'

    def get_permission_object(self):
        return self.request.event

    @cached_property
    def version(self):
        if 'version' in self.kwargs:
            return unquote(self.kwargs['version'])
        else:
            return None

    def dispatch(self, request, *args, **kwargs):
        if 'version' in request.GET:
            if request.resolver_match.url_name.startswith('versioned-'):
                raise Exception('The schedule version can be supplied in the path or querystring, but not both.')

            kwargs['version'] = request.GET['version']
            return HttpResponsePermanentRedirect(reverse(
                f'agenda:versioned-{request.resolver_match.url_name}',
                args=args, kwargs=kwargs
            ))
        else:
            return super().dispatch(request, *args, **kwargs)

    def get_object(self):
        if self.version:
            return self.request.event.schedules.filter(version=self.version).first()
        if self.request.event.current_schedule:
            return self.request.event.current_schedule

    def get_context_data(self, *args, **kwargs):
        ctx = super().get_context_data(*args, **kwargs)
        schedule = self.get_object()
        event = self.request.event

        if not schedule and self.version:
            ctx['version'] = self.version
            ctx['error'] = f'Schedule "{self.version}" not found.'
            return ctx
        elif not schedule:
            ctx['error'] = 'Schedule not found.'
            return ctx
        ctx['schedule'] = schedule
        ctx['schedules'] = event.schedules.filter(published__isnull=False).values_list('version')
        return ctx


class ExporterView(ScheduleDataView):
    permission_required = 'agenda.view_schedule'

    def get_permission_object(self):
        return self.request.event

    def get_exporter(self, request):
        from pretalx.common.signals import register_data_exporters

        url = resolve(request.path_info)
        if url.url_name == 'export':
            exporter = unquote(self.request.GET.get('exporter'))
        else:
            exporter = url.url_name

        responses = register_data_exporters.send(request.event)
        for receiver, response in responses:
            ex = response(request.event)
            if ex.identifier == exporter:
                if ex.public or request.is_orga:
                    return ex

    def get(self, request, *args, **kwargs):
        exporter = self.get_exporter(request)
        if not exporter:
            raise Http404()
        exporter.schedule = self.get_object()
        exporter.is_orga = getattr(self.request, 'is_orga', False)
        file_name, file_type, data = exporter.render()
        resp = HttpResponse(data, content_type=file_type)
        if file_type not in ['application/json', 'text/xml']:
            resp['Content-Disposition'] = f'attachment; filename="{file_name}"'
        return resp


@method_decorator(csp_update(STYLE_SRC="'self' 'unsafe-inline'"), name='dispatch')
class ScheduleView(ScheduleDataView):
    template_name = 'agenda/schedule.html'
    permission_required = 'agenda.view_schedule'

    def get_permission_object(self):
        return self.request.event

    def get_object(self):
        if self.version == 'wip' and self.request.user.has_perm('orga.view_schedule', self.request.event):
            return self.request.event.wip_schedule
        return super().get_object()

    def get_context_data(self, *args, **kwargs):
        from pretalx.schedule.exporters import ScheduleData
        ctx = super().get_context_data(*args, **kwargs)
        ctx['exporters'] = list(exporter(self.request.event) for _, exporter in register_data_exporters.send(self.request.event))
        tz = pytz.timezone(self.request.event.timezone)
        if 'schedule' not in ctx:
            return ctx

        ctx['data'] = ScheduleData(event=self.request.event, schedule=ctx['schedule']).data
        for date in ctx['data']:
            if date.get('first_start') and date.get('last_end'):
                start = date.get('first_start').astimezone(tz).replace(second=0, minute=0)
                end = date.get('last_end').astimezone(tz)
                date['height'] = int((end - start).total_seconds() / 60 * 2)
                date['hours'] = []
                d = start
                while d < end:
                    date['hours'].append(d.strftime('%H:%M'))
                    d += timedelta(hours=1)
                for room in date['rooms']:
                    for talk in room.get('talks', []):
                        talk.top = int((talk.start.astimezone(tz) - start).total_seconds() / 60 * 2)
                        talk.height = int(talk.duration * 2)
                        talk.is_active = talk.start <= now() <= talk.end
        return ctx


class ChangelogView(PermissionRequired, TemplateView):
    template_name = 'agenda/changelog.html'
    permission_required = 'agenda.view_schedule'

    def get_permission_object(self):
        return self.request.event



# Generated by Django 2.0.2 on 2018-03-01 10:58

from django.db import migrations, models
import django.db.models.deletion
import i18nfield.fields
import pretalx.common.mixins.models


class Migration(migrations.Migration):

    dependencies = [
        ('event', '0010_event_plugins'),
        ('person', '0013_auto_20180122_1615'),
    ]

    operations = [
        migrations.CreateModel(
            name='SpeakerInformation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('include_submitters', models.BooleanField(default=False, help_text="Show to every submitter regardless of their submissions' status", verbose_name='Include all submitters')),
                ('exclude_unconfirmed', models.BooleanField(default=False, help_text='Show to speakers only once they have confirmed attendance', verbose_name='Exclude unconfirmed speakers')),
                ('title', i18nfield.fields.I18nCharField(max_length=200, verbose_name='Subject')),
                ('text', i18nfield.fields.I18nTextField(help_text='You can use markdown here.', verbose_name='Text')),
                ('event', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='information', to='event.Event')),
            ],
            bases=(pretalx.common.mixins.models.LogMixin, models.Model),
        ),
    ]



from urllib.parse import urljoin

from django.conf import settings
from django.core.management import call_command
from django.core.management.base import BaseCommand
from django.db import transaction
from django.urls import reverse
from django.utils.translation import ugettext as _


class Command(BaseCommand):
    help = 'Initializes your pretalx instance. Only to be used once.'

    @transaction.atomic
    def handle(self, *args, **options):
        self.stdout.write(self.style.SUCCESS(_('\nWelcome to pretalx! I am the initialization command, and I should be used only once.')))
        self.stdout.write(_('You can abort this command at any time using C-c, and no data will be retained.'))

        self.stdout.write(_('''\nLet\'s get you a user with the right to create new events and access every event on this pretalx instance.'''))

        call_command('createsuperuser')

        url = urljoin(settings.SITE_URL, reverse('orga:dashboard'))
        self.stdout.write(_('\nNow that this is done, you can log in at {url} and get started.').format(url=url))
        self.stdout.write(_('Use the command "import_schedule /path/to/schedule.xml" if you want to import an event."'))



from django.conf import settings

from pretalx.orga.signals import nav_event


def orga_events(request):
    """
    Adds data to all template contexts
    """

    _nav_event = []
    if getattr(request, 'event', None) and hasattr(request, 'user') and request.user.is_authenticated:
        for receiver, response in nav_event.send(request.event, request=request):
            _nav_event += response

    return {
        'nav_event': _nav_event,
        'settings': settings,
    }



import logging

from pretalx.celery_app import app
from pretalx.event.models import Event

logger = logging.getLogger(__name__)


@app.task()
def export_schedule_html(*, event_id: int, make_zip=True):
    from django.core.management import call_command

    event = Event.objects.filter(pk=event_id).first()
    if not event:
        logger.error(f'In export_schedule_html: Could not find Event ID {event_id}')
        return

    cmd = [
        'export_schedule_html',
        event.slug,
    ]

    if make_zip:
        cmd.append('--zip')

    call_command(*cmd)



from rest_framework import viewsets

from pretalx.api.serializers.speaker import (
    SpeakerOrgaSerializer, SpeakerSerializer,
)
from pretalx.person.models import SpeakerProfile


class SpeakerViewSet(viewsets.ReadOnlyModelViewSet):
    serializer_class = SpeakerSerializer
    queryset = SpeakerProfile.objects.none()
    lookup_field = 'user__code__iexact'
    filter_fields = ('user__name', )
    search_fields = ('user__name', )

    def get_serializer_class(self):
        if self.request.user.has_perm('orga.view_speakers', self.request.event):
            return SpeakerOrgaSerializer
        return SpeakerSerializer

    def get_base_queryset(self):
        if self.request.user.has_perm('orga.view_submissions', self.request.event):
            return SpeakerProfile.objects.filter(event=self.request.event)
        if self.request.event.current_schedule and self.request.event.settings.show_schedule:
            return SpeakerProfile.objects.filter(user__submissions__slots__in=self.request.event.current_schedule.talks.all())

    def get_queryset(self):
        return self.get_base_queryset() or self.queryset



import json
from datetime import datetime

from django.utils.translation import ugettext_noop
from hierarkey.models import GlobalSettingsBase, Hierarkey
from i18nfield.strings import LazyI18nString

settings_hierarkey = Hierarkey(attribute_name='settings')


@settings_hierarkey.set_global()
class GlobalSettings(GlobalSettingsBase):
    pass


def i18n_uns(v):
    try:
        return LazyI18nString(json.loads(v))
    except ValueError:
        return LazyI18nString(str(v))


settings_hierarkey.add_type(LazyI18nString,
                            serialize=lambda s: json.dumps(s.data),
                            unserialize=i18n_uns)


settings_hierarkey.add_default('show_schedule', 'True', bool)
settings_hierarkey.add_default('export_html_on_schedule_release', 'True', bool)
settings_hierarkey.add_default('custom_domain', '', str)

settings_hierarkey.add_default('display_header_pattern', '', str)

settings_hierarkey.add_default('allow_override_votes', 'False', bool)
settings_hierarkey.add_default('review_min_score', 0, int)
settings_hierarkey.add_default('review_max_score', 1, int)
settings_hierarkey.add_default('review_score_mandatory', 'False', bool)
settings_hierarkey.add_default('review_text_mandatory', 'False', bool)
settings_hierarkey.add_default('review_deadline', None, datetime)
settings_hierarkey.add_default('review_help_text', LazyI18nString.from_gettext(ugettext_noop("Please give a fair review on why you'd like to see this submission at the conference, or why you think it would not be a good fit.")), LazyI18nString)

settings_hierarkey.add_default('mail_from', 'noreply@example.org', str)
settings_hierarkey.add_default('smtp_use_custom', 'False', bool)
settings_hierarkey.add_default('smtp_host', '', str)
settings_hierarkey.add_default('smtp_port', '587', int)
settings_hierarkey.add_default('smtp_username', '', str)
settings_hierarkey.add_default('smtp_password', '', str)
settings_hierarkey.add_default('smtp_use_tls', 'True', bool)
settings_hierarkey.add_default('smtp_use_ssl', 'False', bool)

settings_hierarkey.add_default('mail_text_reset', LazyI18nString.from_gettext(ugettext_noop("""Hello {name},

you have requested a new password for your submission account at {event}.

To reset your password, click on the following link:

{url}

If this wasn't you, you can just ignore this email.

All the best,
your {event} team.
""")), LazyI18nString)
settings_hierarkey.add_default('mail_on_new_submission', 'False', bool)
settings_hierarkey.add_default('mail_text_new_submission', LazyI18nString.from_gettext(ugettext_noop("""Hi,

you have received a new submission for your event {event_name}:
»{submission_title}« by {speakers}.
You can see details at

  {orga_url}

All the best,
your {event_name} CfP system.
""")), LazyI18nString)

settings_hierarkey.add_default('sent_mail_event_created', 'False', bool)
settings_hierarkey.add_default('sent_mail_cfp_closed', 'False', bool)
settings_hierarkey.add_default('sent_mail_event_over', 'False', bool)
settings_hierarkey.add_default('mail_text_event_created', LazyI18nString.from_gettext(ugettext_noop("""Hi,

we hope you're happy with pretalx as your event's CfP system.
These links may be helpful in the coming days and weeks:

- Your event's dashboard: {event_dashboard}
- A list of submissions: {event_submissions}
- Your schedule editor: {event_schedule}

If there is anything you're missing, come tell us about it
at https://github.com/pretalx/pretalx/issues/new or via an
email to mailto:rixx@cutebit.de!
""")), LazyI18nString)
settings_hierarkey.add_default('mail_text_cfp_closed', LazyI18nString.from_gettext(ugettext_noop("""Hi,

just writing you to let you know that your Call for Participation is now
closed. You'll find a list of all your {submission_count} submissions here:
{event_submissions}

You can add reviewers here: {event_team}
You can review submissions here: {event_review}
And create your schedule here, once you have accepted submissions: {event_schedule}
""")), LazyI18nString)
settings_hierarkey.add_default('mail_text_event_over', LazyI18nString.from_gettext(ugettext_noop("""Hi,

congratulations, your event is over! Hopefully it went well. Here are some
statistics you might find interesting:

- You had {submission_count} talk submissions,
- Of which you selected {talk_count} talks.
- {reviewer_count} reviewers wrote {review_count} reviews.
- You released {schedule_count} schedules in total.
- Over the course of the event, you sent {mail_count} mails.

If there is anything you're missing, come tell us about it
at https://github.com/pretalx/pretalx/issues/new or via an
email to mailto:rixx@cutebit.de!
""")), LazyI18nString)



from django import forms


class ReadOnlyFlag:

    def __init__(self, *args, read_only=False, **kwargs):
        super().__init__(*args, **kwargs)
        self.read_only = read_only
        if read_only:
            for field_name, field in self.fields.items():
                field.disabled = True

    def clean(self):
        if self.read_only:
            raise forms.ValidationError('You are not allowed to submit this data.')
        return super().clean()



from pretalx.common.signals import EventPluginSignal

nav_event = EventPluginSignal(
    providing_args=["request"]
)
"""
This signal allows you to add additional views to the admin panel
navigation. You will get the request as a keyword argument ``request``.
Receivers are expected to return a list of dictionaries. The dictionaries
should contain at least the keys ``label`` and ``url``. You can also return
a fontawesome icon name with the key ``icon``, it will  be respected depending
on the type of navigation. You should also return an ``active`` key with a boolean
set to ``True``, when this item should be marked as active. The ``request`` object
will have an attribute ``event``.

If you use this, you should read the documentation on :ref:`how to deal with URLs <urlconf>`
in pretalx.

As with all plugin signals, the ``sender`` keyword argument will contain the event.
"""



# Generated by Django 2.0.1 on 2018-02-05 17:27

from django.db import migrations
import i18nfield.fields


class Migration(migrations.Migration):

    dependencies = [
        ('schedule', '0010_auto_20171001_0439'),
    ]

    operations = [
        migrations.AddField(
            model_name='room',
            name='speaker_info',
            field=i18nfield.fields.I18nCharField(blank=True, help_text='Information relevant for speakers scheduled in this room, for example room size, special directions, available adapters for video input …', max_length=1000, null=True, verbose_name='Speaker Informtion'),
        ),
        migrations.AlterField(
            model_name='room',
            name='description',
            field=i18nfield.fields.I18nCharField(blank=True, help_text='A description for attendees, for example directions.', max_length=1000, null=True, verbose_name='Description'),
        ),
    ]



import urllib
from contextlib import suppress

from django.core.exceptions import FieldDoesNotExist
from django.db.models import CharField, Q
from django.db.models.functions import Lower
from django.http import Http404
from django.utils.functional import cached_property
from i18nfield.forms import I18nModelForm
from rules.contrib.views import PermissionRequiredMixin

from pretalx.common.forms import SearchForm


class ActionFromUrl:
    write_permission_required = None

    def get_object(self):
        return None

    @cached_property
    def object(self):
        return self.get_object()

    @cached_property
    def permission_object(self):
        return self.object

    @cached_property
    def _action(self):
        if not any(_id in self.kwargs for _id in ['pk', 'code']):
            return 'create'
        if self.request.user.has_perm(self.write_permission_required, self.permission_object):
            return 'edit'
        return 'view'

    def get_context_data(self, *args, **kwargs):
        context = super().get_context_data(*args, **kwargs)
        context['action'] = self._action
        return context

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs['read_only'] = (self._action == 'view')
        if hasattr(self.request, 'event') and issubclass(self.form_class, I18nModelForm):
            kwargs['locales'] = self.request.event.locales
        return kwargs


class Sortable:

    sortable_fields = []

    def sort_queryset(self, qs):
        sort_key = self.request.GET.get('sort') or getattr(self, 'default_sort_field', '')
        if sort_key:
            plain_key = sort_key[1:] if sort_key.startswith('-') else sort_key
            reverse = not (plain_key == sort_key)
            if plain_key in self.sortable_fields:
                is_text = False
                if '__' not in plain_key:
                    with suppress(FieldDoesNotExist):
                        is_text = isinstance(qs.model._meta.get_field(plain_key), CharField)
                else:
                    split_key = plain_key.split('__')
                    if len(split_key) == 2:
                        is_text = isinstance(
                            qs.model._meta.get_field(split_key[0]).related_model._meta.get_field(split_key[1]),
                            CharField
                        )

                if is_text:
                    # TODO: this only sorts direct lookups case insensitively
                    # A sorting field like 'speaker__name' will not be found
                    qs = qs.annotate(key=Lower(plain_key)).order_by('-key' if reverse else 'key')
                else:
                    qs = qs.order_by(sort_key)
        return qs


class Filterable:

    filter_fields = []
    default_filters = []

    def filter_queryset(self, qs):
        self._filter_model = qs.model
        if self.filter_fields:
            for key, value in self.request.GET.items():
                if value:
                    lookup_key = key.split('__')[0]
                    if lookup_key in self.filter_fields:
                        qs = qs.filter(**{key: self.request.GET.get(key)})
        if 'q' in self.request.GET:
            query = urllib.parse.unquote(self.request.GET['q'])
            _filters = [Q(**{field: query}) for field in self.default_filters]
            if len(_filters) > 1:
                _filter = _filters[0]
                for f in _filters[1:]:
                    _filter = _filter | f
                qs = qs.filter(_filter)
            elif len(_filters):
                qs = qs.filter(_filters[0])
        return qs

    def get_context_data(self, *args, **kwargs):
        from django import forms
        ctx = super().get_context_data(*args, **kwargs)
        ctx['search_form'] = SearchForm(self.request.GET if 'q' in self.request.GET else {})
        if hasattr(self, 'filter_form_class'):
            ctx['filter_form'] = self.filter_form_class(self.request.event, self.request.GET)
        elif self.filter_fields:
            ctx['filter_form'] = forms.modelform_factory(self.model, fields=self.filter_fields)(self.request.GET)
            for field in ctx['filter_form'].fields.values():
                field.required = False
                if hasattr(field, 'queryset'):
                    field.queryset = field.queryset.filter(event=self.request.event)
        return ctx


class PermissionRequired(PermissionRequiredMixin):

    def get_login_url(self):
        """ We do this to avoid leaking data about existing pages. """
        raise Http404()



import json
from datetime import datetime, timedelta
from urllib.parse import urlparse

import pytz
import vobject
from django.template.loader import get_template
from django.utils.functional import cached_property
from i18nfield.utils import I18nJSONEncoder

from pretalx.common.exporter import BaseExporter
from pretalx.common.urls import get_base_url
from pretalx.schedule.models import Room


class ScheduleData(BaseExporter):

    def __init__(self, event, schedule=None):
        super().__init__(event)
        self.schedule = schedule

    @cached_property
    def data(self):
        event = self.event
        schedule = self.schedule
        tz = pytz.timezone(event.timezone)

        talks = schedule.talks.filter(is_visible=True)\
            .select_related('submission', 'submission__event', 'room')\
            .prefetch_related('submission__speakers')\
            .order_by('start')
        rooms = Room.objects.filter(pk__in=talks.values_list('room', flat=True).distinct())
        data = [
            {
                'index': index + 1,
                'start': current_date,
                'end': current_date + timedelta(days=1),
                'first_start': min([t.start for t in talks if t.start and t.start.astimezone(tz).date() == current_date.date()] or [0]),
                'last_end': max([t.end for t in talks if t.start and t.start.astimezone(tz).date() == current_date.date()] or [0]),
                'rooms': [{
                    'name': room.name,
                    'talks': [talk for talk in talks
                              if talk.start and talk.start.astimezone(tz).date() == current_date.date() and talk.room_id == room.pk],
                } for room in rooms],
            } for index, current_date in enumerate([
                event.datetime_from + timedelta(days=i) for i in range((event.date_to - event.date_from).days + 1)
            ])
        ]
        return data


class FrabXmlExporter(ScheduleData):
    identifier = 'core-frab-xml'
    verbose_name = 'XML (frab compatible)'
    public = True
    icon = 'fa-code'

    def render(self, **kwargs):
        context = {'data': self.data, 'schedule': self.schedule, 'event': self.event}
        content = get_template('agenda/schedule.xml').render(context=context)
        return f'{self.event.slug}-schedule.xml', 'text/xml', content


class FrabXCalExporter(ScheduleData):
    identifier = 'core-frab-xcal'
    verbose_name = 'XCal (frab compatible)'
    public = True
    icon = 'fa-calendar-o'

    def render(self, **kwargs):
        url = get_base_url(self.event)
        context = {'data': self.data, 'url': url, 'domain': urlparse(url).netloc}
        content = get_template('agenda/schedule.xcal').render(context=context)
        return f'{self.event.slug}.xcal', 'text/xml', content


class FrabJsonExporter(ScheduleData):
    identifier = 'core-frab-json'
    verbose_name = 'JSON (frab compatible)'
    public = True
    icon = '{ }'

    def render(self, **kwargs):
        tz = pytz.timezone(self.event.timezone)
        schedule = self.schedule
        content = {
            'version': schedule.version,
            'conference': {
                'acronym': self.event.slug,
                'title': str(self.event.name),
                'start': self.event.date_from.strftime('%Y-%m-%d'),
                'end': self.event.date_to.strftime('%Y-%m-%d'),
                'daysCount': self.event.duration,
                'timeslot_duration': '00:05',
                'days': [
                    {
                        'index': day['index'],
                        'date': day['start'].strftime('%Y-%m-%d'),
                        'day_start': day['start'].astimezone(tz).isoformat(),
                        'day_end': day['end'].astimezone(tz).isoformat(),
                        'rooms': {
                            str(room['name']): [
                                {
                                    'id': talk.submission.id,
                                    'guid': talk.submission.uuid,
                                    'logo': None,
                                    'date': talk.start.astimezone(tz).isoformat(),
                                    'start': talk.start.astimezone(tz).strftime('%H:%M'),
                                    'duration': talk.export_duration,
                                    'room': str(room['name']),
                                    'slug': talk.submission.code,
                                    'title': talk.submission.title,
                                    'subtitle': '',
                                    'track': None,
                                    'type': str(talk.submission.submission_type.name),
                                    'language': talk.submission.content_locale,
                                    'abstract': talk.submission.abstract,
                                    'description': talk.submission.description,
                                    'recording_license': '',
                                    'do_not_record': talk.submission.do_not_record,
                                    'persons': [
                                        {
                                            'id': person.id,
                                            'name': person.get_display_name(),
                                            'biography': getattr(person.profiles.filter(event=self.event).first(), 'biography', ''),
                                            'answers': [
                                                {
                                                    'question': answer.question.id,
                                                    'answer': answer.answer,
                                                    'options': [option.answer for option in answer.options.all()],
                                                }
                                                for answer in person.answers.all()
                                            ] if getattr(self, 'is_orga', False) else [],
                                        }
                                        for person in talk.submission.speakers.all()
                                    ],
                                    'links': [],
                                    'attachments': [],
                                    'answers': [
                                        {
                                            'question': answer.question.id,
                                            'answer': answer.answer,
                                            'options': [option.answer for option in answer.options.all()],
                                        }
                                        for answer in talk.submission.answers.all()
                                    ] if getattr(self, 'is_orga', False) else [],
                                } for talk in room['talks']
                            ] for room in day['rooms']
                        }

                    } for day in self.data
                ]
            }
        }
        return f'{self.event.slug}.json'.format(self.event.slug), 'application/json', json.dumps({'schedule': content}, cls=I18nJSONEncoder)


class ICalExporter(BaseExporter):
    identifier = 'core-iCal'
    verbose_name = 'iCal'
    public = True
    icon = 'fa-calendar'

    def __init__(self, event, schedule=None):
        super().__init__(event)
        self.schedule = schedule

    def render(self, **kwargs):
        netloc = urlparse(get_base_url(self.event)).netloc
        cal = vobject.iCalendar()
        cal.add('prodid').value = '-//pretalx//{}//'.format(netloc)
        creation_time = datetime.now(pytz.utc)

        talks = self.schedule.talks\
            .filter(is_visible=True)\
            .prefetch_related('submission__speakers')\
            .select_related('submission', 'room')\
            .order_by('start')
        for talk in talks:
            talk.build_ical(cal, creation_time=creation_time, netloc=netloc)

        return f'{self.event.slug}.ics', 'text/calendar', cal.serialize()



from django.contrib import messages
from django.db import transaction
from django.shortcuts import redirect
from django.utils.translation import ugettext_lazy as _
from django.views.generic import TemplateView

from pretalx.common.mixins.views import PermissionRequired


class EventPluginsView(PermissionRequired, TemplateView):
    template_name = 'orga/plugins.html'
    permission_required = 'orga.change_settings'

    def get_object(self):
        return self.request.event

    def get_context_data(self, *args, **kwargs) -> dict:
        from pretalx.common.plugins import get_all_plugins
        context = super().get_context_data(*args, **kwargs)
        context['plugins'] = [p for p in get_all_plugins() if not p.name.startswith('.')]
        context['plugins_active'] = self.request.event.get_plugins()
        return context

    def post(self, request, *args, **kwargs):
        from pretalx.common.plugins import get_all_plugins

        plugins_active = self.request.event.get_plugins()
        plugins_available = {
            p.module: p for p in get_all_plugins()
            if not p.name.startswith('.') and getattr(p, 'visible', True)
        }

        with transaction.atomic():
            for key, value in request.POST.items():
                if key.startswith("plugin:"):
                    module = key.split(":", maxsplit=1)[1]
                    if value == "enable" and module in plugins_available:
                        if hasattr(plugins_available[module].app, 'installed'):
                            getattr(plugins_available[module].app, 'installed')(self.request.event)

                        self.request.event.log_action('pretalx.event.plugins.enabled', person=self.request.user, data={'plugin': module}, orga=True)
                        if module not in plugins_active:
                            plugins_active.append(module)
                    else:
                        self.request.event.log_action('pretalx.event.plugins.disabled', person=self.request.user, data={'plugin': module}, orga=True)
                        if module in plugins_active:
                            plugins_active.remove(module)
            self.request.event.plugins = ','.join(plugins_active)
            self.request.event.save()
            messages.success(self.request, _('Your changes have been saved.'))
        return redirect(self.request.event.orga_urls.plugins)



from django.apps import apps


def get_all_plugins():
    """
    Returns the PretalxPluginMeta classes of all plugins found in the installed Django apps.
    """
    plugins = []
    for app in apps.get_app_configs():
        if hasattr(app, 'PretalxPluginMeta'):
            meta = app.PretalxPluginMeta
            meta.module = app.name
            meta.app = app
            plugins.append(meta)
    return plugins



# Generated by Django 2.0.2 on 2018-03-04 13:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('person', '0014_speakerinformation'),
    ]

    operations = [
        migrations.AddField(
            model_name='speakerinformation',
            name='resource',
            field=models.FileField(blank=True, help_text='Please try to keep your upload small, preferably below 16 MB.', null=True, upload_to='', verbose_name='file'),
        ),
    ]



# Generated by Django 2.0.1 on 2018-02-05 19:50

from django.db import migrations, models
import pretalx.submission.models.submission


class Migration(migrations.Migration):

    dependencies = [
        ('submission', '0019_submissiontype_deadline'),
    ]

    operations = [
        migrations.AddField(
            model_name='submission',
            name='image',
            field=models.ImageField(blank=True, help_text='Optional. Will be displayed publically.', null=True, upload_to=pretalx.submission.models.submission.submission_image_path, verbose_name='Talk image'),
        ),
    ]



from .cfp import CfP
from .feedback import Feedback
from .question import (
    Answer, AnswerOption, Question, QuestionTarget, QuestionVariant,
)
from .resource import Resource
from .review import Review
from .submission import Submission, SubmissionError, SubmissionStates
from .track import Track
from .type import SubmissionType

__all__ = [
    'Answer',
    'AnswerOption',
    'CfP',
    'Feedback',
    'Question',
    'QuestionTarget',
    'QuestionVariant',
    'Resource',
    'Review',
    'Submission',
    'SubmissionError',
    'SubmissionStates',
    'SubmissionType',
    'Track',
]



from urllib.parse import urlparse

import vobject
from csp.decorators import csp_update
from django.conf import settings
from django.core.files.storage import Storage
from django.http import HttpResponse
from django.utils.decorators import method_decorator
from django.views.generic import DetailView

from pretalx.common.mixins.views import PermissionRequired
from pretalx.person.models import SpeakerProfile
from pretalx.submission.models import SubmissionStates


@method_decorator(csp_update(IMG_SRC="https://www.gravatar.com"), name='dispatch')
class SpeakerView(PermissionRequired, DetailView):
    template_name = 'agenda/speaker.html'
    context_object_name = 'profile'
    permission_required = 'agenda.view_speaker'
    slug_field = 'code'

    def get_object(self):
        return SpeakerProfile.objects.filter(
            event=self.request.event, user__code__iexact=self.kwargs['code'],
        ).first()

    def get_context_data(self, object):
        context = super().get_context_data()
        context['speaker'] = object.user
        context['talks'] = object.user.submissions.filter(
            event=self.request.event,
            state=SubmissionStates.CONFIRMED,
            slots__schedule=object.event.current_schedule,
        )
        return context


class SpeakerTalksIcalView(PermissionRequired, DetailView):
    context_object_name = 'profile'
    permission_required = 'agenda.view_speaker'
    slug_field = 'code'

    def get_object(self):
        return SpeakerProfile.objects.filter(
            event=self.request.event, user__code__iexact=self.kwargs['code'],
        ).first()

    def get(self, request, event, **kwargs):
        netloc = urlparse(settings.SITE_URL).netloc
        speaker = self.get_object()
        slots = self.request.event.current_schedule.talks.filter(submission__speakers=speaker.user)

        cal = vobject.iCalendar()
        cal.add('prodid').value = f'-//pretalx//{netloc}//{request.event.slug}//{speaker.code}'

        for slot in slots:
            slot.build_ical(cal)

        resp = HttpResponse(cal.serialize(), content_type='text/calendar')
        speaker_name = Storage().get_valid_name(name=speaker.user.name)
        resp['Content-Disposition'] = f'attachment; filename="{request.event.slug}-{speaker_name}.ics"'
        return resp



# -*- coding: utf-8 -*-
# Generated by Django 1.11.8 on 2017-12-15 22:11
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('person', '0010_eventpermission_review_override_count'),
    ]

    operations = [
        migrations.AddField(
            model_name='speakerprofile',
            name='has_arrived',
            field=models.BooleanField(default=False, verbose_name='The speaker has arrived'),
        ),
    ]



from typing import Any, Callable, List, Tuple

import django.dispatch
from django.apps import apps
from django.conf import settings
from django.dispatch.dispatcher import NO_RECEIVERS

from pretalx.event.models import Event

app_cache = {}


def _populate_app_cache():
    global app_cache
    apps.check_apps_ready()
    for ac in apps.app_configs.values():
        app_cache[ac.name] = ac


class EventPluginSignal(django.dispatch.Signal):
    """
    This is an extension to Django's built-in signals which differs in a way that it sends
    out it's events only to receivers which belong to plugins that are enabled for the given
    Event.
    """

    def _is_active(self, sender, receiver):
        if sender is None:
            # Send to all events!
            return True

        # Find the Django application this belongs to
        searchpath = receiver.__module__
        core_module = any([searchpath.startswith(cm) for cm in settings.LOCAL_APPS])
        app = None
        if not core_module:
            while True:
                app = app_cache.get(searchpath)
                if "." not in searchpath or app:
                    break
                searchpath, _ = searchpath.rsplit(".", 1)

        # Only fire receivers from active plugins and core modules
        if core_module or (sender and app and app.name in sender.get_plugins()):
            return True
        return False

    def send(self, sender: Event, **named) -> List[Tuple[Callable, Any]]:
        """
        Send signal from sender to all connected receivers that belong to
        plugins enabled for the given Event.

        sender is required to be an instance of ``pretalx.event.models.Event``.
        """
        if sender and not isinstance(sender, Event):
            raise ValueError("Sender needs to be an event.")

        responses = []
        if not self.receivers or self.sender_receivers_cache.get(sender) is NO_RECEIVERS:
            return responses

        if not app_cache:
            _populate_app_cache()

        for receiver in self._live_receivers(sender):
            if self._is_active(sender, receiver):
                response = receiver(signal=self, sender=sender, **named)
                responses.append((receiver, response))
        return sorted(responses, key=lambda r: (receiver.__module__, receiver.__name__))

    def send_chained(self, sender: Event, chain_kwarg_name, **named) -> List[Tuple[Callable, Any]]:
        """
        Send signal from sender to all connected receivers. The return value of the first receiver
        will be used as the keyword argument specified by ``chain_kwarg_name`` in the input to the
        second receiver and so on. The return value of the last receiver is returned by this method.

        sender is required to be an instance of ``pretalx.event.models.Event``.
        """
        if sender and not isinstance(sender, Event):
            raise ValueError("Sender needs to be an event.")

        response = named.get(chain_kwarg_name)
        if not self.receivers or self.sender_receivers_cache.get(sender) is NO_RECEIVERS:
            return response

        if not app_cache:
            _populate_app_cache()

        for receiver in self._live_receivers(sender):
            if self._is_active(sender, receiver):
                named[chain_kwarg_name] = response
                response = receiver(signal=self, sender=sender, **named)
        return response


periodic_task = django.dispatch.Signal()
"""
This is a regular django signal (no pretalx event signal) that we send out every
time the periodic task cronjob runs. This interval is not sharply defined, it can
be everything between a minute and a day. The actions you perform should be
idempotent, i.e. it should not make a difference if this is sent out more often
than expected.
"""

register_data_exporters = EventPluginSignal(
    providing_args=[]
)
"""
This signal is sent out to get all known data exporters. Receivers should return a
subclass of pretalx.common.exporter.BaseExporter

As with all event plugin signals, the ``sender`` keyword argument will contain the event.
"""



import rules

from pretalx.submission.models.submission import SubmissionStates


@rules.predicate
def is_orga(user, obj):
    if not user or user.is_anonymous or not obj:
        return False
    from pretalx.person.models import EventPermission
    return user.is_administrator or EventPermission.objects.filter(
        user=user, event=obj.event, is_orga=True
    ).exists()


@rules.predicate
def is_reviewer(user, obj):
    if not user or user.is_anonymous or not obj:
        return False
    from pretalx.person.models import EventPermission
    return user.is_administrator or EventPermission.objects.filter(
        user=user, event=obj.event, is_reviewer=True
    ).exists()


@rules.predicate
def is_administrator(user, obj):
    return user.is_administrator


@rules.predicate
def person_can_view_information(user, obj):
    event = obj.event
    submissions = event.submissions.filter(speakers__in=[user])
    if obj.include_submitters:
        return submissions.exists()
    if obj.exclude_unconfirmed:
        return submissions.filter(state=SubmissionStates.CONFIRMED).exists()
    return submissions.filter(state__in=[SubmissionStates.CONFIRMED, SubmissionStates.ACCEPTED]).exists()


rules.add_perm('person.view_information', is_orga | person_can_view_information)



from django.apps import AppConfig


class ScheduleConfig(AppConfig):
    name = 'pretalx.schedule'

    def ready(self):
        from . import signals  # noqa


default_app_config = 'pretalx.schedule.ScheduleConfig'



import random
from datetime import timedelta

from django.contrib import messages
from django.db import transaction
from django.shortcuts import get_object_or_404, redirect
from django.utils.crypto import get_random_string
from django.utils.functional import cached_property
from django.utils.timezone import now
from django.utils.translation import override, ugettext as _
from django.views.generic import ListView, TemplateView, View

from pretalx.common.mixins.views import (
    ActionFromUrl, Filterable, PermissionRequired, Sortable,
)
from pretalx.common.urls import build_absolute_uri
from pretalx.common.views import CreateOrUpdateView
from pretalx.mail.models import QueuedMail
from pretalx.orga.forms import SubmissionForm
from pretalx.person.models import SpeakerProfile, User
from pretalx.submission.forms import SubmissionFilterForm
from pretalx.submission.models import (
    Question, Submission, SubmissionError, SubmissionStates,
)


def create_user_as_orga(email, submission=None):
    if not email:
        return

    nick = email.split('@')[0].lower()
    while User.objects.filter(nick__iexact=nick).exists():
        nick += random.choice([
            '_1', '_2', '_11', '_42', '_the_first', '_the_third',
            '_speaker', '_third_of_their_name', '_', '123', nick
        ])

    user = User.objects.create_user(
        nick=nick,
        password=get_random_string(32),
        email=email.lower(),
        pw_reset_token=get_random_string(32),
        pw_reset_time=now() + timedelta(days=7),
    )
    with override(submission.content_locale):
        invitation_link = build_absolute_uri('cfp:event.recover', kwargs={'event': submission.event.slug, 'token': user.pw_reset_token})
        invitation_text = _('''Hi!

You have been set as the speaker of a submission to the Call for Participation
of {event}, titled »{title}«. An account has been created for you – please follow
this link to set your account password.

{invitation_link}

Afterwards, you can edit your user profile and see the state of your submission.

The {event} orga crew''').format(event=submission.event.name, title=submission.title, invitation_link=invitation_link)
        QueuedMail.objects.create(
            event=submission.event,
            to=user.email,
            reply_to=submission.event.email,
            subject=str(_('You have been added to a submission for {event}').format(event=submission.event.name)),
            text=invitation_text,
        )
    return user


class SubmissionViewMixin(PermissionRequired):

    def get_object(self):
        return get_object_or_404(
            self.request.event.submissions,
            code__iexact=self.kwargs.get('code'),
        )

    @cached_property
    def object(self):
        return self.get_object()

    def get_context_data(self, *args, **kwargs):
        ctx = super().get_context_data(*args, **kwargs)
        ctx['submission'] = self.get_object()
        return ctx


class SubmissionStateChange(SubmissionViewMixin, TemplateView):
    permission_required = 'orga.change_submission_state'
    template_name = 'orga/submission/state_change.html'
    TARGETS = {
        'submit': SubmissionStates.SUBMITTED,
        'accept': SubmissionStates.ACCEPTED,
        'reject': SubmissionStates.REJECTED,
        'confirm': SubmissionStates.CONFIRMED,
        'delete': SubmissionStates.DELETED,
        'withdraw': SubmissionStates.WITHDRAWN,
        'cancel': SubmissionStates.CANCELED,
    }

    @cached_property
    def target(self) -> str:
        """ Returns one of submit|accept|reject|confirm|delete|withdraw|cancel """
        return self.TARGETS[self.request.resolver_match.url_name.split('.')[-1]]

    @cached_property
    def is_allowed(self):
        return self.target in SubmissionStates.valid_next_states[self.object.state]

    def do(self, force=False):
        method = getattr(self.object, SubmissionStates.method_names[self.target])
        try:
            method(person=self.request.user, force=force, orga=True)
        except SubmissionError as e:
            messages.error(self.request, e.message)

    def get_success_url(self):
        next_url = self.request.POST.get('next', '')

        if next_url == self.object.event.orga_urls.reviews:
            return next_url

        return self.object.orga_urls.base

    @transaction.atomic
    def post(self, request, *args, **kwargs):
        if self.is_allowed:
            self.do()
        else:
            self.do(force=True)
        return redirect(self.get_success_url())

    def get_context_data(self, *args, **kwargs):
        ctx = super().get_context_data(*args, **kwargs)
        ctx['target'] = self.target
        ctx['next'] = self.request.GET.get('next')
        return ctx


class SubmissionSpeakersAdd(SubmissionViewMixin, View):
    permission_required = 'submission.edit_speaker_list'

    def dispatch(self, request, *args, **kwargs):
        super().dispatch(request, *args, **kwargs)
        submission = self.get_object()
        nick = request.POST.get('nick')
        try:
            if '@' in nick:
                speaker = User.objects.get(email__iexact=nick)
            else:
                speaker = User.objects.get(nick__iexact=nick)
        except User.DoesNotExist:
            speaker = create_user_as_orga(request.POST.get('nick'), submission=submission)
        if not speaker:
            messages.error(request, _('Please provide a valid nick or email address!'))
        else:
            if submission not in speaker.submissions.all():
                speaker.submissions.add(submission)
                speaker.save(update_fields=['submissions'])
                submission.log_action('pretalx.submission.speakers.add', person=request.user, orga=True)
                messages.success(request, _('The speaker has been added to the submission.'))
            else:
                messages.warning(request, _('The speaker was already part of the submission.'))
        if not speaker.profiles.filter(event=request.event).exists():
            SpeakerProfile.objects.create(user=speaker, event=request.event)
        return redirect(submission.orga_urls.speakers)


class SubmissionSpeakersDelete(SubmissionViewMixin, View):
    permission_required = 'submission.edit_speaker_list'

    def dispatch(self, request, *args, **kwargs):
        super().dispatch(request, *args, **kwargs)
        submission = self.get_object()
        speaker = get_object_or_404(User, nick__iexact=request.GET.get('nick'))

        if submission in speaker.submissions.all():
            speaker.submissions.remove(submission)
            speaker.save(update_fields=['submissions'])
            submission.log_action('pretalx.submission.speakers.remove', person=request.user, orga=True)
            messages.success(request, _('The speaker has been removed from the submission.'))
        else:
            messages.warning(request, _('The speaker was not part of this submission.'))
        return redirect(submission.orga_urls.speakers)


class SubmissionSpeakers(SubmissionViewMixin, TemplateView):
    template_name = 'orga/submission/speakers.html'
    permission_required = 'submission.view_submission'

    def get_context_data(self, *args, **kwargs):
        context = super().get_context_data(*args, **kwargs)
        submission = context['submission']
        context['speakers'] = [{
                'id': speaker.id,
                'name': speaker.get_display_name(),
                'nick': speaker.nick,
                'biography': speaker.profiles.get_or_create(event=submission.event)[0].biography,
                'other_submissions': speaker.submissions.filter(event=submission.event).exclude(code=submission.code),
            } for speaker in submission.speakers.all()
        ]
        context['users'] = User.objects.all()  # TODO: yeah, no
        return context


class SubmissionQuestions(SubmissionViewMixin, TemplateView):
    template_name = 'orga/submission/answer_list.html'
    permission_required = 'submission.view_submission'

    def get_context_data(self, *args, **kwargs):
        context = super().get_context_data(*args, **kwargs)
        submission = context['submission']
        answers = [
            question.answers.filter(submission=submission).first()
            for question in Question.all_objects.filter(event=submission.event, target='submission')
        ]
        context.update({
            'answer_list': [a for a in answers if a],
            'submission': submission,
        })
        return context


class SubmissionContent(ActionFromUrl, SubmissionViewMixin, CreateOrUpdateView):
    model = Submission
    form_class = SubmissionForm
    template_name = 'orga/submission/content.html'
    permission_required = 'submission.view_submission'

    @property
    def write_permission_required(self):
        if 'code' in self.kwargs:
            return 'submission.edit_submission'
        return 'orga.create_submission'

    def get_permission_required(self):
        if 'code' in self.kwargs:
            return ['submission.view_submission']
        return ['orga.create_submission']

    def get_permission_object(self):
        return self.get_object() or self.request.event

    def get_object(self):
        return self.request.event.submissions.filter(code__iexact=self.kwargs.get('code')).first()

    def get_success_url(self) -> str:
        self.kwargs.update({'code': self.object.code})
        return self.object.orga_urls.base

    def form_valid(self, form):
        created = invited = not self.object
        form.instance.event = self.request.event
        ret = super().form_valid(form)
        self.object = form.instance

        if created:
            email = form.cleaned_data['speaker']
            try:
                if '@' in email:
                    speaker = User.objects.get(email__iexact=email)
                else:
                    speaker = User.objects.get(nick__iexact=email)
                invited = False
            except User.DoesNotExist:
                speaker = create_user_as_orga(email=email, submission=form.instance)

            form.instance.speakers.add(speaker)

        if form.has_changed():
            action = 'pretalx.submission.' + ('create' if created else 'update')
            form.instance.log_action(action, person=self.request.user, orga=True)
        if created and invited:
            messages.success(self.request, _('The submission has been created and the speaker has been invited to add an account!'))
        elif created:  # TODO: send email!
            messages.success(self.request, _('The submission has been created; the speaker already had an account on this system.'))
        else:
            messages.success(self.request, _('The submission has been updated!'))
        return ret

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs['event'] = self.request.event
        return kwargs


class SubmissionList(PermissionRequired, Sortable, Filterable, ListView):
    model = Submission
    context_object_name = 'submissions'
    template_name = 'orga/submission/list.html'
    default_filters = ('code__icontains', 'speakers__name__icontains', 'speakers__nick__icontains', 'title__icontains')
    filter_fields = ('submission_type', 'state')
    filter_form_class = SubmissionFilterForm
    sortable_fields = ('code', 'title', 'submission_type', 'state')
    permission_required = 'orga.view_submissions'
    paginate_by = 25

    def get_permission_object(self):
        return self.request.event

    def get_queryset(self):
        qs = self.request.event.submissions.select_related('submission_type').order_by('-id').all()
        qs = self.filter_queryset(qs)
        qs = self.sort_queryset(qs)
        return qs


class FeedbackList(SubmissionViewMixin, ListView):
    template_name = 'orga/submission/feedback_list.html'
    context_object_name = 'feedback'
    paginate_by = 25
    permission_required = 'submission.view_feedback'

    def get_queryset(self):
        return self.get_object().feedback.all().order_by('pk')



# Generated by Django 2.0.2 on 2018-03-05 22:30

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('mail', '0003_auto_20171001_1358'),
        ('event', '0010_event_plugins'),
    ]

    operations = [
        migrations.AddField(
            model_name='event',
            name='question_template',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='mail.MailTemplate'),
        ),
    ]



from .information import SpeakerInformation
from .permission import EventPermission
from .profile import SpeakerProfile
from .user import User

__all__ = [
    'EventPermission',
    'SpeakerInformation',
    'SpeakerProfile',
    'User',
]



from django import forms

from pretalx.submission.models import Resource


class ResourceForm(forms.ModelForm):

    class Meta:
        model = Resource
        fields = [
            'resource', 'description',
        ]



from django.apps import AppConfig


class SubmissionConfig(AppConfig):
    name = 'pretalx.submission'

    def ready(self):
        from . import permissions  # noqa


default_app_config = 'pretalx.submission.SubmissionConfig'



from django.conf.urls import include, url

from pretalx.event.models.event import SLUG_CHARS

from .views import feed, location, schedule, speaker, talk


def get_schedule_urls(regex_prefix, name_prefix=""):
    """
    given a prefix (e.g. /schedule) generate matching schedule-ruls (e.g. /schedule.json, /schedule/feed.xml, ...)
    """

    regex_prefix = regex_prefix.rstrip('/')

    return [
        url(f"{regex_prefix}{regex}", view, name=f"{name_prefix}{name}")
        for regex, view, name in [
            ('/$', schedule.ScheduleView.as_view(), 'schedule'),
            ('.xml$', schedule.ExporterView.as_view(), 'core-frab-xml'),
            ('.xcal$', schedule.ExporterView.as_view(), 'core-frab-xcal'),
            ('.json$', schedule.ExporterView.as_view(), 'core-frab-json'),
            ('.ics$', schedule.ExporterView.as_view(), 'core-iCal'),
            ('/export$', schedule.ExporterView.as_view(), 'export'),
            ('/feed.xml$', feed.ScheduleFeed(), 'feed'),
        ]
    ]


app_name = 'agenda'
urlpatterns = [
    url(f'^(?P<event>[{SLUG_CHARS}]+)/', include([
        url('^schedule/changelog$', schedule.ChangelogView.as_view(), name='schedule.changelog'),
        *get_schedule_urls('^schedule'),
        *get_schedule_urls('^schedule/v/(?P<version>.+)', 'versioned-'),
        url('^location/$', location.LocationView.as_view(), name='location'),
        url('^talk/(?P<slug>\w+)/$', talk.TalkView.as_view(), name='talk'),
        url('^talk/(?P<slug>\w+)/feedback/$', talk.FeedbackView.as_view(), name='feedback'),
        url('^talk/(?P<slug>\w+).ics$', talk.SingleICalView.as_view(), name='ical'),
        url('^speaker/(?P<code>\w+)/$', speaker.SpeakerView.as_view(), name='speaker'),
        url('^speaker/(?P<code>\w+)/talks.ics$', speaker.SpeakerTalksIcalView.as_view(), name='speaker.talks.ical'),
    ])),
]



from .feedback import FeedbackForm
from .question import QuestionsForm
from .resource import ResourceForm
from .submission import InfoForm, SubmissionFilterForm

__all__ = [
    'FeedbackForm',
    'InfoForm',
    'QuestionsForm',
    'ResourceForm',
    'SubmissionFilterForm',
]



import logging
import os

from csp.decorators import csp_update
from django.conf import settings
from django.contrib import messages
from django.contrib.auth import login
from django.core.files.storage import FileSystemStorage
from django.forms import ValidationError
from django.shortcuts import redirect
from django.urls import reverse
from django.utils.crypto import get_random_string
from django.utils.decorators import method_decorator
from django.utils.translation import ugettext_lazy as _
from django.views import View
from formtools.wizard.views import NamedUrlSessionWizardView

from pretalx.cfp.views.event import EventPageMixin
from pretalx.common.mail import SendMailException
from pretalx.common.phrases import phrases
from pretalx.mail.context import template_context_from_submission
from pretalx.mail.models import MailTemplate
from pretalx.person.forms import SpeakerProfileForm, UserForm
from pretalx.person.models import User
from pretalx.submission.forms import InfoForm, QuestionsForm
from pretalx.submission.models import Answer, QuestionTarget, QuestionVariant

FORMS = [
    ("info", InfoForm),
    ("questions", QuestionsForm),
    ("user", UserForm),
    ("profile", SpeakerProfileForm),
]

TEMPLATES = {
    "info": "cfp/event/submission_info.html",
    "questions": "cfp/event/submission_questions.html",
    "user": "cfp/event/submission_user.html",
    "profile": "cfp/event/submission_profile.html",
}


class SubmitStartView(EventPageMixin, View):
    def get(self, request, *args, **kwargs):
        newid = get_random_string(length=6)
        return redirect(reverse('cfp:event.submit', kwargs={
            'event': request.event.slug,
            'step': 'info',
            'tmpid': newid
        }))


def show_questions_page(wizard):
    return wizard.request.event.questions.all().exists()


def show_user_page(wizard):
    return not wizard.request.user.is_authenticated


@method_decorator(csp_update(SCRIPT_SRC="'self' 'unsafe-inline'"), name='dispatch')
class SubmitWizard(EventPageMixin, NamedUrlSessionWizardView):
    form_list = FORMS
    condition_dict = {
        'questions': show_questions_page,
        'user': show_user_page
    }
    file_storage = FileSystemStorage(os.path.join(settings.MEDIA_ROOT, 'avatars'))

    def get(self, request, *args, **kwargs):
        if not request.event.cfp.is_open:
            messages.error(request, phrases.cfp.submissions_closed)
            return redirect(reverse('cfp:event.start', kwargs={'event': request.event.slug}))
        return super().get(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        if not request.event.cfp.is_open:
            messages.error(request, phrases.cfp.submissions_closed)
            return redirect(reverse('cfp:event.start', kwargs={'event': request.event.slug}))
        return super().post(request, *args, **kwargs)

    def get_form_instance(self, step):
        return super().get_form_instance(step)

    def get_form_kwargs(self, step=None):
        kwargs = super().get_form_kwargs(step)
        if step in ['info', 'profile', 'questions']:
            kwargs['event'] = self.request.event
        if step == 'profile':
            user_data = self.get_cleaned_data_for_step('user')
            if user_data and user_data.get('user_id'):
                kwargs['user'] = User.objects.filter(pk=user_data['user_id']).first()
            if not kwargs.get('user') and self.request.user.is_authenticated:
                kwargs['user'] = self.request.user
            kwargs['read_only'] = False
            kwargs['essential_only'] = True
        if step == 'questions':
            kwargs['target'] = ''
        return kwargs

    def get_template_names(self):
        return [TEMPLATES[self.steps.current]]

    def get_prefix(self, request, *args, **kwargs):
        return super().get_prefix(request, *args, **kwargs) + ':' + kwargs.get('tmpid')

    def get_step_url(self, step):
        return reverse(self.url_name, kwargs={
            'step': step,
            'tmpid': self.kwargs.get('tmpid'),
            'event': self.kwargs.get('event')
        })

    def _handle_question_answer(self, sub, qid, value, user=None):
        question = self.request.event.questions.filter(pk=qid).first()
        if not question:
            return
        if question.target == QuestionTarget.SUBMISSION:
            answer = Answer(question=question, submission=sub)
        elif question.target == QuestionTarget.SPEAKER:
            answer = Answer(question=question, person=user)

        if question.variant == QuestionVariant.MULTIPLE:
            answstr = ", ".join([str(o) for o in value])
            answer.save()
            if value:
                answer.answer = answstr
                answer.options.add(*value)
        elif question.variant == QuestionVariant.CHOICES:
            answer.save()
            if value:
                answer.options.add(value)
                answer.answer = value.answer
        else:
            answer.answer = value
        answer.save()

    def done(self, form_list, form_dict, **kwargs):
        if self.request.user.is_authenticated:
            user = self.request.user
        else:
            uid = form_dict['user'].save()
            user = User.objects.filter(pk=uid).first()
        if not user:
            raise ValidationError(_('There was an error with your user account. Please contact the organiser for further help.'))

        form_dict['info'].instance.event = self.request.event
        form_dict['info'].save()
        form_dict['info'].instance.speakers.add(user)
        sub = form_dict['info'].instance
        form_dict['profile'].user = user
        form_dict['profile'].save()
        if 'questions' in form_dict:
            for k, value in form_dict['questions'].cleaned_data.items():
                qid = k.split('_')[1]
                self._handle_question_answer(sub, qid, value, user=user)

        try:
            sub.event.ack_template.to_mail(
                user=user, event=self.request.event, context=template_context_from_submission(sub),
                skip_queue=True, locale=user.locale
            )
            if self.request.event.settings.mail_on_new_submission:
                MailTemplate(
                    event=self.request.event,
                    subject=_('[{event}] New submission!').format(event=self.request.event.slug),
                    text=self.request.event.settings.mail_text_new_submission,
                ).to_mail(
                    user=self.request.event.email, event=self.request.event,
                    context=template_context_from_submission(sub),
                    skip_queue=True, locale=self.request.event.locale
                )
        except SendMailException as e:
            logging.getLogger('').warning(str(e))
            messages.warning(self.request, phrases.cfp.submission_email_fail)

        sub.log_action('pretalx.submission.create', person=user)
        messages.success(self.request, phrases.cfp.submission_success)
        login(self.request, user, backend='django.contrib.auth.backends.ModelBackend')
        return redirect(reverse('cfp:event.user.submissions', kwargs={
            'event': self.request.event.slug
        }))



import os

from bakery.views import BuildableDetailView
from django.conf import settings

from pretalx.agenda.views.schedule import ExporterView, ScheduleView
from pretalx.agenda.views.speaker import SpeakerView
from pretalx.agenda.views.talk import SingleICalView, TalkView
from pretalx.person.models import SpeakerProfile
from pretalx.schedule.models import Schedule
from pretalx.submission.models import Submission


class PretalxExportContextMixin():
    def __init__(self, *args, **kwargs):
        self._exporting_event = kwargs.pop('_exporting_event', None)

        if not self._exporting_event:
            raise Exception('Use the provided "export_schedule_html" management command to export the HTML schedule.')

        super().__init__(*args, **kwargs)

    def create_request(self, *args, **kwargs):
        request = super().create_request(*args, **kwargs)
        request.event = self._exporting_event
        return request

    def get_context_data(self, *args, **kwargs):
        self.object = self.get_object()  # ScheduleView crashes without this

        ctx = super().get_context_data(*args, **kwargs)
        ctx['is_html_export'] = True
        return ctx

    def get_url(self, obj):
        return obj.urls.public

    def get_queryset(self):
        qs = super().get_queryset()
        return qs.filter(event=self._exporting_event)

    def get_file_build_path(self, obj):
        dir_path, file_name = os.path.split(self.get_url(obj))

        path = os.path.join(settings.BUILD_DIR, dir_path[1:])
        os.path.exists(path) or os.makedirs(path)

        return os.path.join(path, file_name)


# current schedule
class ExportScheduleView(PretalxExportContextMixin, BuildableDetailView, ScheduleView):
    queryset = Schedule.objects.filter(published__isnull=False).order_by('published')

    def get_url(self, obj):
        return obj.event.urls.schedule


class ExportFrabXmlView(PretalxExportContextMixin, BuildableDetailView, ExporterView):
    queryset = Schedule.objects.filter(published__isnull=False).order_by('published')

    def get_url(self, obj):
        return obj.event.urls.frab_xml

    def get_content(self):
        return self.get(self.request, self._exporting_event).content

    def get_build_path(self, obj):
        return self.get_file_build_path(obj)


class ExportFrabXCalView(PretalxExportContextMixin, BuildableDetailView, ExporterView):
    queryset = Schedule.objects.filter(published__isnull=False).order_by('published')

    def get_url(self, obj):
        return obj.event.urls.frab_xcal

    def get_content(self):
        return self.get(self.request, self._exporting_event).content

    def get_build_path(self, obj):
        return self.get_file_build_path(obj)


class ExportFrabJsonView(PretalxExportContextMixin, BuildableDetailView, ExporterView):
    queryset = Schedule.objects.filter(published__isnull=False).order_by('published')

    def get_url(self, obj):
        return obj.event.urls.frab_json

    def get_content(self):
        return self.get(self.request, self._exporting_event).content

    def get_build_path(self, obj):
        return self.get_file_build_path(obj)


class ExportICalView(PretalxExportContextMixin, BuildableDetailView, ExporterView):
    queryset = Schedule.objects.filter(published__isnull=False).order_by('published')

    def get_url(self, obj):
        return obj.event.urls.ical

    def get_content(self):
        return self.get(self.request, self._exporting_event).content

    def get_build_path(self, obj):
        return self.get_file_build_path(obj)


# all schedule versions
class ExportScheduleVersionsView(PretalxExportContextMixin, BuildableDetailView, ScheduleView):
    queryset = Schedule.objects.filter(version__isnull=False)


class ExportTalkView(PretalxExportContextMixin, BuildableDetailView, TalkView):
    queryset = Submission.objects.filter(slots__schedule__published__isnull=False).distinct()


class ExportTalkICalView(PretalxExportContextMixin, BuildableDetailView, SingleICalView):
    queryset = Submission.objects.filter(slots__schedule__published__isnull=False).distinct()

    def get_url(self, obj):
        return obj.urls.ical

    def get_content(self):
        return self.get(self.request, self._exporting_event).content

    def get_build_path(self, obj):
        return self.get_file_build_path(obj)


class ExportSpeakerView(PretalxExportContextMixin, BuildableDetailView, SpeakerView):
    queryset = SpeakerProfile.objects.filter(user__submissions__slots__schedule__published__isnull=False).distinct()



# Generated by Django 2.0.1 on 2018-01-22 22:14

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('person', '0011_speakerprofile_has_arrived'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='is_administrator',
            field=models.BooleanField(default=False),
        ),
    ]



import os.path
from shutil import make_archive

from bakery.management.commands.build import Command as BakeryBuildCommand
from django.conf import settings
from django.core.management.base import CommandError
from django.test import override_settings
from django.urls import get_callable
from django.utils import translation

from pretalx.event.models import Event


class Command(BakeryBuildCommand):
    help = 'Exports event schedule as a static HTML dump'

    def add_arguments(self, parser):
        super().add_arguments(parser)
        parser.add_argument('event', type=str)
        parser.add_argument('--zip', action='store_true')

    @classmethod
    def get_output_dir(cls, event):
        return os.path.join(settings.HTMLEXPORT_ROOT, event.slug)

    @classmethod
    def get_output_zip_path(cls, event):
        return cls.get_output_dir(event) + '.zip'

    def handle(self, *args, **options):
        try:
            event = Event.objects.get(slug__iexact=options['event'])
        except Event.DoesNotExist:
            raise CommandError('Could not find event with slug "{}"'.format(options['event']))

        self._exporting_event = event
        translation.activate(event.locale)

        with override_settings(COMPRESS_ENABLED=True, COMPRESS_OFFLINE=True, BUILD_DIR=self.get_output_dir(event)):
            super().handle(*args, **options)
            if options.get('zip'):
                make_archive(settings.BUILD_DIR, 'zip', settings.BUILD_DIR, settings.BUILD_DIR)

    def build_views(self):
        for view_str in self.view_list:
            view = get_callable(view_str)
            view(_exporting_event=self._exporting_event).build_method()



from django.db import models
from django.utils.translation import ugettext_lazy as _
from i18nfield.fields import I18nCharField, I18nTextField

from pretalx.common.mixins import LogMixin
from pretalx.common.urls import EventUrls


class SpeakerInformation(LogMixin, models.Model):
    event = models.ForeignKey(
        to='event.Event',
        related_name='information',
        on_delete=models.CASCADE,
    )
    include_submitters = models.BooleanField(
        verbose_name=_('Include all submitters'),
        help_text=_('Show to every submitter regardless of their submissions\' status'),
        default=False,
    )
    exclude_unconfirmed = models.BooleanField(
        verbose_name=_('Exclude unconfirmed speakers'),
        help_text=_('Show to speakers only once they have confirmed attendance'),
        default=False,
    )
    title = I18nCharField(
        verbose_name=_('Subject'),
        max_length=200,
    )
    text = I18nTextField(
        verbose_name=_('Text'),
        help_text=_('You can use markdown here.'),
    )
    resource = models.FileField(
        verbose_name=_('file'),
        null=True, blank=True,
        help_text=_('Please try to keep your upload small, preferably below 16 MB.'),
    )

    class orga_urls(EventUrls):
        base = edit = '{self.event.orga_urls.information}/{self.pk}'
        delete = '{base}/delete/'



from django import forms
from django.utils.translation import ugettext_lazy as _

from pretalx.mail.models import QueuedMail


class SubmissionInvitationForm(forms.Form):
    speaker = forms.EmailField(label=_('Speaker E-Mail'))
    subject = forms.CharField(label=_('Subject'))
    text = forms.CharField(widget=forms.Textarea(), label=_('Text'))

    def __init__(self, submission, speaker, *args, **kwargs):
        self.submission = submission
        initial = kwargs.get('initial', {})
        initial['subject'] = _('[{event}] {speaker} invites you to join their talk!').format(
            event=submission.event.slug, speaker=speaker.name or speaker.nick,
        )
        initial['text'] = _('''Hi!

I'd like to invite you to be a speaker in my talk »{title}«
at {event}. Please follow this link to join:

  {url}

I'm looking forward to it!
{speaker}''').format(
            event=submission.event.name, title=submission.title,
            url=submission.urls.accept_invitation.full(),
            speaker=speaker.name or speaker.nick,
        )
        super().__init__(*args, **kwargs)

    def save(self):
        QueuedMail(
            event=self.submission.event,
            to=self.cleaned_data['speaker'],
            subject=self.cleaned_data['subject'],
            text=self.cleaned_data['text'],
        ).send()



from typing import Tuple


class BaseExporter:
    """
    This is the base class for all data exporters
    """

    def __init__(self, event):
        self.event = event

    def __str__(self):
        return self.identifier

    @property
    def verbose_name(self) -> str:
        """
        A human-readable name for this exporter. This should be short but
        self-explaining. Good examples include 'JSON' or 'Microsoft Excel'.
        """
        raise NotImplementedError()  # NOQA

    @property
    def identifier(self) -> str:
        """
        A short and unique identifier for this exporter.
        This should only contain lowercase letters and in most
        cases will be the same as your package name.
        """
        raise NotImplementedError()  # NOQA

    @property
    def public(self) -> str:
        """
        Return True if the exported data should be publicly available once the event is public, False otherwise.
        """
        raise NotImplementedError()  # NOQA

    @property
    def icon(self) -> str:
        """
        Return either a fa- string or some other symbol to accompany the exporter in displays.
        """
        raise NotImplementedError()  # NOQA

    def render(self, **kwargs) -> Tuple[str, str, str]:
        """
        Render the exported file and return a tuple consisting of a filename, a file type
        and file content.
        """
        raise NotImplementedError()  # NOQA



import string

from django.db import models
from django.utils.crypto import get_random_string
from django.utils.translation import ugettext_lazy as _

from pretalx.common.mixins import LogMixin
from pretalx.common.urls import EventUrls, build_absolute_uri


class EventPermission(LogMixin, models.Model):
    event = models.ForeignKey(
        to='event.Event',
        related_name='permissions',
        on_delete=models.CASCADE,
    )
    user = models.ForeignKey(
        to='person.User',
        related_name='permissions',
        on_delete=models.CASCADE,
        null=True, blank=True,
    )
    is_orga = models.BooleanField(
        default=True,
        verbose_name=_('Organizes the event'),
    )
    is_reviewer = models.BooleanField(
        default=False,
        verbose_name=_('May write reviews for this event'),
    )
    review_override_count = models.PositiveIntegerField(
        default=0,
        verbose_name=_('Override votes for reviews'),
        help_text=_('How many times may this user cast an overriding votes or vetos?'),
    )
    invitation_token = models.CharField(
        max_length=255,
        null=True, blank=True,
    )
    invitation_email = models.EmailField(
        null=True, blank=True,
    )

    def send_invite_email(self):
        from pretalx.mail.models import QueuedMail

        role = _('organiser')
        if not self.is_orga and self.is_reviewer:
            role = _('reviewer')
        elif self.is_orga and self.is_reviewer:
            role = _('organiser and reviewer')
        if self.user:
            invitation_link = build_absolute_uri('orga:event.dashboard', event=self.event, kwargs={'event': self.event.slug})
            invitation_text = _('''Hi!
You have been added to the {event} team as a {role} - you can access the
organiser area here:

    {invitation_link}

See you there,
The {event} crew''').format(role=role, event=self.event.name, invitation_link=invitation_link)
            invitation_subject = _('You have been granted additional privileges on {event}').format(event=self.event.name)
            to = self.user.email
        else:
            self.invitation_token = get_random_string(allowed_chars=string.ascii_lowercase + string.digits, length=20)
            self.save()

            invitation_link = build_absolute_uri('orga:invitation.view', event=self.event, kwargs={'code': self.invitation_token})
            invitation_text = _('''Hi!
You have been invited to the {event} team as a {role} - Please click here to accept:

    {invitation_link}

See you there,
The {event} crew (minus you)''').format(role=role, event=self.event.name, invitation_link=invitation_link)
            invitation_subject = _('You have been invited to the {event} crew').format(event=self.event.name)
            to = self.invitation_email

        return QueuedMail(
            to=to, reply_to=self.event.email,
            subject=str(invitation_subject), text=str(invitation_text),
            event=self.event,
        )

    class urls(EventUrls):
        invitation = '/orga/invitation/{self.invitation_token}'

    def __str__(self):
        user = getattr(self.user, 'nick', self.invitation_email)
        return f'EventPermission(event={self.event.slug}, user={user}, is_orga={self.is_orga}, is_reviewer={self.is_reviewer}'



from django.contrib.auth.mixins import LoginRequiredMixin
from django.urls import reverse
from django.views.generic import TemplateView

from pretalx.common.mixins.views import PermissionRequired
from pretalx.event.models import Event


class EventPageMixin(PermissionRequired):
    permission_required = 'cfp.view_event'

    def get_permission_object(self):
        return getattr(self.request, 'event', None)


class LoggedInEventPageMixin(EventPageMixin, LoginRequiredMixin):

    def get_login_url(self) -> str:
        return reverse('cfp:event.login', kwargs={
            'event': self.request.event.slug
        })


class EventStartpage(EventPageMixin, TemplateView):
    template_name = 'cfp/event/index.html'


class EventCfP(EventPageMixin, TemplateView):
    template_name = 'cfp/event/cfp.html'


class GeneralView(TemplateView):
    template_name = 'cfp/index.html'

    def get_context_data(self, *args, **kwargs):
        ctx = super().get_context_data(*args, **kwargs)
        ctx['events'] = Event.objects.filter(is_public=True)
        ctx['orga_events'] = [e for e in Event.objects.filter(is_public=False) if self.request.user.has_perm('cfp.view_event', e)]
        return ctx



import importlib
from contextlib import suppress

from django.apps import apps
from django.conf import settings
from django.conf.urls import include, url
from django.conf.urls.static import static

plugin_patterns = []
for app in apps.get_app_configs():
    if hasattr(app, 'PretalxPluginMeta'):
        if importlib.util.find_spec(app.name + '.urls'):
            urlmod = importlib.import_module(app.name + '.urls')
            single_plugin_patterns = []
            if hasattr(urlmod, 'urlpatterns'):
                single_plugin_patterns += urlmod.urlpatterns
            plugin_patterns.append(
                url(r'', include((single_plugin_patterns, app.label)))
            )

urlpatterns = [
    url(r'^orga/', include('pretalx.orga.urls', namespace='orga')),
    url(r'^api/', include('pretalx.api.urls', namespace='api')),
    url(r'', include('pretalx.agenda.urls', namespace='agenda')),
    url(r'', include('pretalx.cfp.urls', namespace='cfp')),
    url(r'', include((plugin_patterns, 'plugins'))),
]

if settings.DEBUG:
    with suppress(ImportError):
        import debug_toolbar
        urlpatterns += [
            url(r'^__debug__/', include(debug_toolbar.urls)),
        ]
    urlpatterns += static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)



from django.contrib.auth.password_validation import validate_password
from django.forms import CharField

from pretalx.common.forms.widgets import (
    PasswordConfirmationInput, PasswordStrengthInput,
)


class GlobalValidator:

    def __call__(self, value):
        return validate_password(value)


class PasswordField(CharField):
    default_validators = [GlobalValidator()]

    def __init__(self, *args, **kwargs):
        if 'widget' not in kwargs:
            kwargs['widget'] = PasswordStrengthInput(render_value=False)
        super().__init__(*args, **kwargs)


class PasswordConfirmationField(CharField):

    def __init__(self, *args, **kwargs):
        if 'widget' not in kwargs:
            kwargs['widget'] = PasswordConfirmationInput(confirm_with=kwargs.pop('confirm_with', None))
        super().__init__(*args, **kwargs)



from django.apps import AppConfig


class EventConfig(AppConfig):
    name = 'pretalx.event'

    def ready(self):
        from . import services  # noqa


default_app_config = 'pretalx.event.EventConfig'



from rest_framework.serializers import (
    CharField, ModelSerializer, SerializerMethodField,
)

from pretalx.api.serializers.question import AnswerSerializer
from pretalx.person.models import SpeakerProfile, User
from pretalx.submission.models import Answer


class SubmitterSerializer(ModelSerializer):
    biography = SerializerMethodField()

    def get_biography(self, obj):
        if self.context.get('request') and self.context['request'].event:
            profile = obj.profiles.filter(event=self.context['request'].event).first()
            if profile:
                return profile.biography
        return ''

    class Meta:
        model = User
        fields = (
            'code', 'name', 'biography'
        )


class SpeakerSerializer(ModelSerializer):
    code = CharField(source='user.code')
    name = CharField(source='user.name')
    submissions = SerializerMethodField()

    def get_submissions(self, obj):
        talks = obj.event.current_schedule.talks.all() if obj.event.current_schedule else []
        return obj.user.submissions\
            .filter(event=obj.event, slots__in=talks)\
            .values_list('code', flat=True)

    class Meta:
        model = SpeakerProfile
        fields = (
            'code', 'name', 'biography', 'submissions',
        )


class SpeakerOrgaSerializer(SpeakerSerializer):
    answers = AnswerSerializer(Answer.objects.all(), many=True, read_only=True)

    def get_submissions(self, obj):
        return obj.user.submissions\
            .filter(event=obj.event)\
            .values_list('code', flat=True)

    class Meta(SpeakerSerializer.Meta):
        fields = SpeakerSerializer.Meta.fields + ('answers', )



# Generated by Django 2.0.2 on 2018-02-06 17:20

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('submission', '0020_submission_image'),
    ]

    operations = [
        migrations.AddField(
            model_name='answer',
            name='review',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='answers', to='submission.Review'),
        ),
    ]



from django.contrib import messages
from django.shortcuts import get_object_or_404, redirect
from django.urls import reverse
from django.utils.functional import cached_property
from django.views.generic import ListView, View

from pretalx.common.mixins.views import (
    ActionFromUrl, Filterable, PermissionRequired, Sortable,
)
from pretalx.common.views import CreateOrUpdateView
from pretalx.person.forms import (
    SpeakerFilterForm, SpeakerInformationForm, SpeakerProfileForm,
)
from pretalx.person.models import SpeakerInformation, SpeakerProfile, User
from pretalx.submission.models.submission import SubmissionStates


class SpeakerList(PermissionRequired, Sortable, Filterable, ListView):
    model = SpeakerProfile
    template_name = 'orga/speaker/list.html'
    context_object_name = 'speakers'
    default_filters = ('user__nick__icontains', 'user__email__icontains', 'user__name__icontains')
    sortable_fields = ('user__nick', 'user__email', 'user__name')
    default_sort_field = 'user__name'
    paginate_by = 25
    permission_required = 'orga.view_speakers'

    def get_permission_object(self):
        return self.request.event

    def get_context_data(self, *args, **kwargs):
        ctx = super().get_context_data(*args, **kwargs)
        ctx['filter_form'] = SpeakerFilterForm()
        return ctx

    def get_queryset(self):
        qs = SpeakerProfile.objects.filter(event=self.request.event)
        qs = self.filter_queryset(qs)
        if 'role' in self.request.GET:
            # TODO: this returns speakers accepted for *other* events. SubQuery time
            if self.request.GET['role'] == 'true':
                qs = qs.filter(user__submissions__state__in=[SubmissionStates.ACCEPTED, SubmissionStates.CONFIRMED])
            elif self.request.GET['role'] == 'false':
                qs = qs.exclude(user__submissions__state__in=[SubmissionStates.ACCEPTED, SubmissionStates.CONFIRMED])
        qs = self.sort_queryset(qs)
        return qs


class SpeakerDetail(PermissionRequired, ActionFromUrl, CreateOrUpdateView):
    template_name = 'orga/speaker/form.html'
    form_class = SpeakerProfileForm
    model = User
    permission_required = 'orga.view_speaker'
    write_permission_required = 'orga.change_speaker'

    def get_object(self):
        return get_object_or_404(
            User.objects.filter(submissions__in=self.request.event.submissions.all()).order_by('id').distinct(),
            pk=self.kwargs['pk'],
        )

    @cached_property
    def object(self):
        return self.get_object()

    def get_permission_object(self):
        return self.get_object().profiles.filter(event=self.request.event).first()

    @cached_property
    def permission_object(self):
        return self.get_permission_object()

    def get_success_url(self) -> str:
        return reverse('orga:speakers.view', kwargs={'event': self.request.event.slug, 'pk': self.kwargs['pk']})

    def get_context_data(self, *args, **kwargs):
        from pretalx.submission.models import QuestionTarget
        ctx = super().get_context_data(*args, **kwargs)
        submissions = self.request.event.submissions.filter(speakers__in=[self.object])
        ctx['submission_count'] = submissions.count()
        ctx['submissions'] = submissions
        ctx['questions'] = [{
            'question': question,
            'answers': question.answers.filter(person=self.object)
        } for question in self.request.event.questions.filter(target__in=[QuestionTarget.SUBMISSION, QuestionTarget.SPEAKER])]
        ctx['questions'] = [q for q in ctx['questions'] if q['answers'].count() and any(a.answer is not None for a in q['answers'])]
        return ctx

    def form_valid(self, form):
        messages.success(self.request, 'The speaker profile has been updated.')
        if form.has_changed():
            profile = self.object.profiles.filter(event=self.request.event).first()
            if profile:
                profile.log_action('pretalx.user.profile.update', person=self.request.user, orga=True)
        return super().form_valid(form)

    def get_form_kwargs(self, *args, **kwargs):
        ret = super().get_form_kwargs(*args, **kwargs)
        ret.update({
            'event': self.request.event,
            'user': self.get_object(),
        })
        return ret


class SpeakerToggleArrived(PermissionRequired, View):
    permission_required = 'orga.change_speaker'

    def get_object(self):
        return get_object_or_404(SpeakerProfile, event=self.request.event, user_id=self.kwargs['pk'])

    def dispatch(self, request, event, pk):
        profile = self.get_object()
        profile.has_arrived = not profile.has_arrived
        profile.save()
        action = 'pretalx.speaker.arrived' if profile.has_arrived else 'pretalx.speaker.unarrived'
        profile.user.log_action(action, data={'event': self.request.event.slug}, person=self.request.user, orga=True)
        if request.GET.get('from') == 'list':
            return redirect(reverse('orga:speakers.list', kwargs={'event': self.kwargs['event']}))
        return redirect(reverse(
            'orga:speakers.view',
            kwargs=self.kwargs,
        ))


class InformationList(PermissionRequired, ListView):
    model = SpeakerInformation
    template_name = 'orga/speaker/information_list.html'
    context_object_name = 'information'
    paginate_by = 25
    permission_required = 'orga.view_information'

    def get_permission_object(self):
        return self.request.event


class InformationDetail(PermissionRequired, ActionFromUrl, CreateOrUpdateView):
    template_name = 'orga/speaker/information_form.html'
    form_class = SpeakerInformationForm
    model = SpeakerInformation
    permission_required = 'orga.view_information'
    write_permission_required = 'orga.change_information'

    def get_permission_object(self):
        return self.request.event

    def get_object(self):
        if 'pk' in self.kwargs:
            return self.request.event.information.filter(pk=self.kwargs['pk']).first()

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs.pop('read_only', None)
        return kwargs

    def form_valid(self, form):
        if not hasattr(form.instance, 'event') or not form.instance.event:
            form.instance.event = self.request.event
        return super().form_valid(form)

    def get_success_url(self):
        return self.request.event.orga_urls.information


class InformationDelete(PermissionRequired, View):
    write_permission_required = 'orga.change_information'



from datetime import timedelta

from django.dispatch import receiver
from django.utils.timezone import now

from pretalx.celery_app import app
from pretalx.common.signals import periodic_task
from pretalx.event.models import Event


@app.task()
def task_periodic_event_services(event_slug):
    event = Event.objects.filter(slug=event_slug).prefetch_related('_settings_objects', 'submissions__slots').first()
    _now = now()
    if not event:
        return

    if not event.settings.sent_mail_event_created:
        if timedelta(0) <= (_now - event.log_entries.last().timestamp) <= timedelta(days=1):
            event.send_orga_mail(event.settings.mail_text_event_created)
            event.settings.sent_mail_event_created = True

    if not event.settings.sent_mail_cfp_closed and event.cfp.deadline:
        if timedelta(0) <= (_now - event.cfp.deadline) <= timedelta(days=1):
            event.send_orga_mail(event.settings.mail_text_cfp_closed)
            event.settings.sent_mail_cfp_closed = True

    if not event.settings.sent_mail_event_over:
        if (_now.date() + timedelta(days=3)) < (_now.date() + timedelta(days=1)) > event.date_to:
            if event.current_schedule and event.current_schedule.talks.filter(is_visible=True).count():
                event.send_orga_mail(event.settings.mail_text_event_over, stats=True)
                event.settings.sent_mail_event_over = True


@receiver(periodic_task)
def periodic_event_services(sender, **kwargs):
    for event in Event.objects.all().values_list('slug', flat=True):
        task_periodic_event_services.apply_async(args=(event, ))



from datetime import timedelta


def serialize_duration(minutes):
    duration = timedelta(minutes=minutes)
    days = duration.days
    hours = int(duration.total_seconds() // 3600 - days * 24)
    minutes = int(duration.seconds // 60 % 60)
    fmt = f'{minutes:02}'
    if hours or days:
        fmt = f'{hours:02}:{fmt}'
        if days:
            fmt = f'{days}:{fmt}'
    else:
        fmt = f'00:{fmt}'
    return fmt



from csp.decorators import csp_update
from django import forms
from django.contrib import messages
from django.contrib.auth import login
from django.core.exceptions import PermissionDenied
from django.db import transaction
from django.shortcuts import get_object_or_404, redirect
from django.urls import reverse
from django.utils.decorators import method_decorator
from django.utils.functional import cached_property
from django.utils.translation import ugettext_lazy as _
from django.views.generic import FormView, TemplateView
from rest_framework.authtoken.models import Token

from pretalx.common.mixins.views import ActionFromUrl, PermissionRequired
from pretalx.common.phrases import phrases
from pretalx.common.tasks import regenerate_css
from pretalx.common.views import CreateOrUpdateView
from pretalx.event.models import Event
from pretalx.orga.forms import EventForm, EventSettingsForm
from pretalx.orga.forms.event import MailSettingsForm
from pretalx.person.forms import LoginInfoForm, OrgaProfileForm, UserForm
from pretalx.person.models import EventPermission, User


class EventSettingsPermission(PermissionRequired):
    permission_required = 'orga.change_settings'

    def get_permission_object(self):
        return self.request.event


class EventDetail(ActionFromUrl, CreateOrUpdateView):
    model = Event
    form_class = EventForm
    write_permission_required = 'orga.change_settings'

    @cached_property
    def _action(self):
        if 'event' not in self.kwargs:
            return 'create'
        if self.request.user.has_perm(self.write_permission_required, self.object):
            return 'edit'
        return 'view'

    def get_template_names(self):
        if self._action == 'create':
            return 'orga/settings/create_event.html'
        return 'orga/settings/form.html'

    def dispatch(self, request, *args, **kwargs):
        if self._action == 'create':
            if not request.user.is_anonymous and not request.user.is_administrator:
                raise PermissionDenied()
        else:
            if not request.user.has_perm('orga.change_settings', self.object):
                raise PermissionDenied()
        return super().dispatch(request, *args, **kwargs)

    def get_object(self):
        return self.object

    @cached_property
    def object(self):
        try:
            return self.request.event
        except AttributeError:
            return

    @cached_property
    def sform(self):
        if not hasattr(self.request, 'event') or not self.request.event:
            return
        return EventSettingsForm(
            read_only=(self._action == 'view'),
            locales=self.request.event.locales,
            obj=self.request.event,
            attribute_name='settings',
            data=self.request.POST if self.request.method == "POST" else None,
            prefix='settings'
        )

    def get_context_data(self, *args, **kwargs):
        context = super().get_context_data(*args, **kwargs)
        context['sform'] = self.sform
        context['url_placeholder'] = f'https://{self.request.host}/'
        return context

    def get_success_url(self) -> str:
        return self.object.orga_urls.settings

    def form_valid(self, form):
        new_event = not bool(form.instance.pk)
        if not new_event:
            if not self.sform.is_valid():
                return self.form_invalid(form)

        ret = super().form_valid(form)

        if new_event:
            messages.success(self.request, _('Yay, a new event! Check these settings and configure a CfP and you\'re good to go!'))
            form.instance.log_action('pretalx.event.create', person=self.request.user, orga=True)
            EventPermission.objects.create(
                event=form.instance,
                user=self.request.user,
                is_orga=True,
            )
        else:
            self.sform.save()
            form.instance.log_action('pretalx.event.update', person=self.request.user, orga=True)
            messages.success(self.request, _('The event settings have been saved.'))
        regenerate_css.apply_async(args=(form.instance.pk,))
        return ret


class EventMailSettings(EventSettingsPermission, ActionFromUrl, FormView):
    form_class = MailSettingsForm
    template_name = 'orga/settings/mail.html'
    write_permission_required = 'orga.change_settings'

    def get_success_url(self) -> str:
        return self.request.event.orga_urls.mail_settings

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs['obj'] = self.request.event
        kwargs['attribute_name'] = 'settings'
        kwargs['locales'] = self.request.event.locales
        return kwargs

    def form_valid(self, form):
        form.save()

        if self.request.POST.get('test', '0').strip() == '1':
            backend = self.request.event.get_mail_backend(force_custom=True)
            try:
                backend.test(self.request.event.settings.mail_from)
            except Exception as e:
                messages.warning(self.request, _('An error occured while contacting the SMTP server: %s') % str(e))
                return redirect(self.request.event.orga_urls.mail_settings)
            else:
                if form.cleaned_data.get('smtp_use_custom'):
                    messages.success(self.request, _('Yay, your changes have been saved and the connection attempt to '
                                                     'your SMTP server was successful.'))
                else:
                    messages.success(self.request, _('We\'ve been able to contact the SMTP server you configured. '
                                                     'Remember to check the "use custom SMTP server" checkbox, '
                                                     'otherwise your SMTP server will not be used.'))
        else:
            messages.success(self.request, _('Yay! We saved your changes.'))

        ret = super().form_valid(form)
        return ret


@method_decorator(csp_update(SCRIPT_SRC="'self' 'unsafe-inline'"), name='get')
class EventTeam(EventSettingsPermission, TemplateView):
    template_name = 'orga/settings/team.html'

    @cached_property
    def formset(self):
        formset_class = forms.inlineformset_factory(
            Event, EventPermission, can_delete=True, extra=0,
            fields=[
                'is_orga',
                'is_reviewer',
                'review_override_count',
                'invitation_email',
            ],
        )
        return formset_class(
            self.request.POST if self.request.method == 'POST' else None,
            queryset=EventPermission.objects.filter(event=self.request.event),
            instance=self.request.event,
        )

    def get_context_data(self, *args, **kwargs):
        ctx = super().get_context_data(*args, **kwargs)
        ctx['formset'] = self.formset
        return ctx

    @classmethod
    def _find_user(cls, email):
        from pretalx.person.models import User
        return User.objects.filter(nick=email).first() or User.objects.filter(email=email).first()

    @transaction.atomic
    def post(self, request, *args, **kwargs):
        if not self.formset.is_valid():
            messages.error(request, phrases.base.error_saving_changes)
            return redirect(self.request.event.orga_urls.team_settings)
        permissions = self.formset.save(commit=False)
        mails = []

        for permission in self.formset.deleted_objects:
            permission.delete()

        for permission in permissions:
            if permission.invitation_email:
                user = self._find_user(permission.invitation_email)

                if user:
                    permission.user = user
                    permission.invitation_email = None
                    permission.invitation_token = None
                mails.append(permission.send_invite_email())
                request.event.log_action('pretalx.invite.orga.send', person=request.user, orga=True)
                messages.success(
                    request,
                    _('<{email}> has been invited to your team - more team members help distribute the workload, so … yay!').format(email=permission.invitation_email)
                )

            permission.save()

        for permission in permissions:
            if permission.user:
                EventPermission.objects \
                    .filter(event=permission.event, user=permission.user) \
                    .exclude(id=permission.id) \
                    .delete()

        for mail in mails:
            mail.send()

        return redirect(self.request.event.orga_urls.team_settings)


@method_decorator(csp_update(SCRIPT_SRC="'self' 'unsafe-inline'"), name='dispatch')
class InvitationView(FormView):
    template_name = 'orga/invitation.html'
    form_class = UserForm

    @property
    def object(self):
        return get_object_or_404(
            EventPermission,
            invitation_token__iexact=self.kwargs.get('code'),
            user__isnull=True,
        )

    def get_context_data(self, *args, **kwargs):
        ctx = super().get_context_data(*args, **kwargs)
        ctx['invitation'] = self.object
        return ctx

    def form_valid(self, form):
        form.save()
        permission = self.object
        user = User.objects.filter(pk=form.cleaned_data.get('user_id')).first()
        if not user:
            messages.error(self.request, _('There was a problem with your authentication. Please contact the organiser for further help.'))
            return redirect(self.request.event.urls.base)

        perm = EventPermission.objects.filter(user=user, event=permission.event).exclude(pk=permission.pk).first()
        event = permission.event

        if perm:
            if permission.is_orga:
                perm.is_orga = True
            if permission.is_reviewer:
                perm.is_reviewer = True
            perm.save()
            permission.delete()
            permission = perm

        permission.user = user
        permission.save()
        permission.event.log_action('pretalx.invite.orga.accept', person=user, orga=True)
        messages.info(self.request, _('You are now part of the event team!'))

        login(self.request, user, backend='django.contrib.auth.backends.ModelBackend')
        return redirect(event.orga_urls.base)


@method_decorator(csp_update(SCRIPT_SRC="'self' 'unsafe-inline'"), name='dispatch')
class UserSettings(TemplateView):
    form_class = LoginInfoForm
    template_name = 'orga/user.html'

    def get_success_url(self) -> str:
        return reverse('orga:user.view')

    @cached_property
    def login_form(self):
        return LoginInfoForm(user=self.request.user,
                             data=(self.request.POST
                                   if self.request.method == 'POST' and self.request.POST.get('form') == 'login'
                                   else None))

    @cached_property
    def profile_form(self):
        return OrgaProfileForm(instance=self.request.user,
                               data=(self.request.POST
                                     if self.request.method == 'POST' and self.request.POST.get('form') == 'profile'
                                     else None))

    def get_context_data(self, **kwargs):
        ctx = super().get_context_data(**kwargs)
        ctx['token'] = Token.objects.filter(user=self.request.user).first() or Token.objects.create(user=self.request.user)
        ctx['login_form'] = self.login_form
        ctx['profile_form'] = self.profile_form
        return ctx

    def post(self, request, *args, **kwargs):
        if self.login_form.is_bound:
            if self.login_form.is_valid():
                self.login_form.save()
                messages.success(request, _('Your changes have been saved.'))
                request.user.log_action('pretalx.user.password.update')
                return redirect(self.get_success_url())
        elif self.profile_form.is_bound:
            if self.profile_form.is_valid():
                self.profile_form.save()
                messages.success(request, _('Your changes have been saved.'))
                request.user.log_action('pretalx.user.profile.update')
                return redirect(self.get_success_url())

        messages.error(self.request, _('Oh :( We had trouble saving your input. See below for details.'))
        return super().get(request, *args, **kwargs)



from django.contrib import messages
from django.db import models
from django.shortcuts import get_object_or_404, redirect
from django.utils.functional import cached_property
from django.utils.translation import ugettext_lazy as _
from django.views.generic import ListView, TemplateView

from pretalx.common.mixins.views import PermissionRequired
from pretalx.common.phrases import phrases
from pretalx.common.views import CreateOrUpdateView
from pretalx.orga.forms import ReviewForm
from pretalx.person.models import EventPermission
from pretalx.submission.forms import QuestionsForm
from pretalx.submission.models import Review, SubmissionStates


class ReviewDashboard(PermissionRequired, ListView):
    template_name = 'orga/review/dashboard.html'
    paginate_by = 25
    context_object_name = 'submissions'
    permission_required = 'orga.view_review_dashboard'

    def get_queryset(self, *args, **kwargs):
        overridden_reviews = Review.objects.filter(override_vote__isnull=False, submission_id=models.OuterRef('pk'))
        return self.request.event.submissions\
            .filter(state__in=[SubmissionStates.SUBMITTED, SubmissionStates.ACCEPTED, SubmissionStates.REJECTED, SubmissionStates.CONFIRMED])\
            .order_by('review_id')\
            .annotate(has_override=models.Exists(overridden_reviews))\
            .annotate(avg_score=models.Case(
                models.When(
                    has_override=True,
                    then=self.request.event.settings.review_max_score + 1,
                ),
                default=models.Avg('reviews__score')
            ))\
            .order_by('-state', '-avg_score', 'code')

    def get_permission_object(self):
        return self.request.event

    def get_context_data(self, *args, **kwargs):
        ctx = super().get_context_data(*args, **kwargs)
        missing_reviews = Review.find_missing_reviews(self.request.event, self.request.user)
        reviewers = EventPermission.objects.filter(is_reviewer=True, event=self.request.event)
        ctx['missing_reviews'] = missing_reviews
        ctx['next_submission'] = missing_reviews.first()
        ctx['reviewers'] = reviewers.count()
        ctx['active_reviewers'] = reviewers.filter(user__reviews__isnull=False).order_by('user__id').distinct().count()
        ctx['review_count'] = self.request.event.reviews.count()
        if ctx['active_reviewers'] > 1:
            ctx['avg_reviews'] = round(ctx['review_count'] / ctx['active_reviewers'], 1)
        return ctx


class ReviewSubmission(PermissionRequired, CreateOrUpdateView):

    form_class = ReviewForm
    model = Review
    template_name = 'orga/submission/review.html'
    permission_required = 'submission.view_reviews'

    @cached_property
    def submission(self):
        return get_object_or_404(
            self.request.event.submissions,
            code__iexact=self.kwargs['code'],
        )

    @cached_property
    def object(self):
        return self.submission.reviews.exclude(user__in=self.submission.speakers.all()).filter(user=self.request.user).first()

    def get_object(self):
        return self.object

    def get_permission_object(self):
        return self.submission

    @cached_property
    def read_only(self):
        return not self.request.user.has_perm('submission.review_submission', self.get_object() or self.submission)

    @cached_property
    def qform(self):
        return QuestionsForm(
            target='reviewer', event=self.request.event,
            data=(self.request.POST if self.request.method == 'POST' else None),
            files=(self.request.FILES if self.request.method == 'POST' else None),
            speaker=self.request.user,
            review=self.object,
            readonly=self.read_only,
        )

    def get_context_data(self, *args, **kwargs):
        ctx = super().get_context_data(*args, **kwargs)
        ctx['submission'] = self.submission
        ctx['review'] = self.object
        ctx['read_only'] = self.read_only
        ctx['override_left'] = self.request.user.remaining_override_votes(self.request.event)
        ctx['qform'] = self.qform
        ctx['reviews'] = [{
                'score': review.display_score,
                'text': review.text,
                'user': review.user.get_display_name(),
                'answers': [
                    review.answers.filter(question=question).first()
                    for question in self.qform.queryset
                ]
            } for review in self.submission.reviews.exclude(pk=(self.object.pk if self.object else None))
        ]
        return ctx

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs['event'] = self.request.event
        kwargs['user'] = self.request.user
        kwargs['read_only'] = self.read_only
        return kwargs

    def form_valid(self, form):
        if not self.qform.is_valid():
            messages.error(self.request, _('There have been erros with your input.'))
            return redirect(self.get_success_url())
        form.instance.submission = self.submission
        form.instance.user = self.request.user
        if not form.instance.pk:
            if not self.request.user.has_perm('submission.review_submission', self.submission):
                messages.error(self.request, _('You cannot review this submission at this time.'))
                return redirect(self.get_success_url())
        if form.instance.pk and not self.request.user.has_perm('submission.edit_review', form.instance):
            messages.error(self.request, _('You cannot review this submission at this time.'))
            return redirect(self.get_success_url())
        form.save()
        self.qform.review = form.instance
        self.qform.save()
        return super().form_valid(form)

    def get_success_url(self) -> str:
        if self.request.POST.get('show_next', '0').strip() == '1':
            next_submission = Review.find_missing_reviews(self.request.event, self.request.user).first()
            if next_submission:
                messages.success(self.request, phrases.orga.another_review)
                return next_submission.orga_urls.reviews
            else:
                messages.success(self.request, _('Nice, you have no submissions left to review!'))
                return self.request.event.orga_urls.reviews

        return self.submission.orga_urls.reviews


class ReviewSubmissionDelete(PermissionRequired, TemplateView):
    template_name = 'orga/review/submission_delete.html'
    permission_required = 'orga.remove_review'

    def get_permission_object(self):
        return self.request.event
